Components.utils.import("resource://gre/modules/ctypes.jsm");    

const OSStatus = ctypes.int32_t;
const CFIndex = ctypes.long;
const OptionBits = ctypes.uint32_t;

var CoreFoundation = {
  lib: null,      /* The Core Foundation library reference */
  
  init: function() {
    Application.console.log("Opening CoreFoundation");
    this.lib = ctypes.open("/System/Library/Frameworks/CoreFoundation.framework/CoreFoundation");
    
    // CFRange
    
    Application.console.log("Declaring CFRange stuff");
    
    this.CFRange = new ctypes.StructType("CFRange", [{'location': ctypes.int32_t}, {'length': ctypes.int32_t}]);
    
    // CFString

    Application.console.log("Declaring CFString stuff");
    this.CFStringRef = new ctypes.PointerType("CFStringRef");
    
    this.CFStringCreateWithCharacters = this.lib.declare("CFStringCreateWithCharacters",
                                ctypes.default_abi,
                                this.CFStringRef,         // returns a new CFStringRef
                                ctypes.voidptr_t,         // allocator
                                ctypes.jschar.ptr,        // pointer to the Unicode string
                                ctypes.int32_t);          // length of the string
    
    this.CFStringGetLength = this.lib.declare("CFStringGetLength",
                                ctypes.default_abi,
                                ctypes.int32_t,           // returns the number of Unicode characters
                                this.CFStringRef);        // the string to check
                                
    this.CFStringGetCharacters = this.lib.declare("CFStringGetCharacters",
                                ctypes.default_abi,
                                ctypes.void_t,
                                this.CFStringRef,         // the string to get characters from
                                this.CFRange,             // the range of characters
                                ctypes.voidptr_t);        // pointer to buffer to receive chars
    
    // Generic CFType routines
    
    Application.console.log("Declaring CFType stuff");
    
    this.CFRelease = this.lib.declare("CFRelease",
                                ctypes.default_abi,
                                ctypes.void_t,
                                ctypes.voidptr_t);        // input: object to release
    
    this.CFShow = this.lib.declare("CFShow",
                                ctypes.default_abi,
                                ctypes.void_t,            // returns nothing
                                ctypes.voidptr_t);        // input: CFType data to output
    
    this.CFCopyDescription = this.lib.declare("CFCopyDescription",
                                ctypes.default_abi,
                                this.CFStringRef,         // returns a CFStringRef
                                ctypes.voidptr_t);        // input: a CFType object
    
    // CFURL routines
    
    Application.console.log("Declaring CFURL stuff");
    
    this.CFURLRef = new ctypes.PointerType("CFURLRef");
    
    this.CFURLCreateWithString = this.lib.declare("CFURLCreateWithString",
                                ctypes.default_abi,
                                this.CFURLRef,            // returns CFURLRef
                                ctypes.voidptr_t,         // allocator
                                this.CFStringRef,         // string to convert to URL
                                this.CFURLRef);           // base URL
    
    this.CFURLGetFSRef = this.lib.declare("CFURLGetFSRef",
                                ctypes.default_abi,
                                ctypes.bool,              // Returns a bool
                                this.CFURLRef,            // input: URL to convert
                                ctypes.voidptr_t);        // input: Pointer to FSRef to fill
    
    // CFArray
    
    Application.console.log("Declaring CFArray stuff");
    this.CFArrayRef = new ctypes.PointerType("CFArrayRef");
    
    this.CFArrayGetValueAtIndex = this.lib.declare("CFArrayGetValueAtIndex",
                                ctypes.default_abi,
                                ctypes.voidptr_t,         // returns the value
                                ctypes.voidptr_t,         // input: the array
                                CFIndex);                 // input: index
    
    // CFMutableArray
    
    Application.console.log("Declaring CFMutableArray stuff");
    this.CFMutableArrayRef = new ctypes.PointerType("CFMutableArrayRef");
    
    this.CFArrayCreateMutable = this.lib.declare("CFArrayCreateMutable",
                                ctypes.default_abi,
                                this.CFMutableArrayRef,   // returns an array
                                ctypes.voidptr_t,         // allocator
                                ctypes.int32_t,           // capacity
                                ctypes.voidptr_t);        // CFArrayCallbacks *callbacks
                                
    this.CFArrayAppendValue = this.lib.declare("CFArrayAppendValue",
                                ctypes.default_abi,
                                ctypes.void_t,
                                this.CFMutableArrayRef,    // the array to append to
                                ctypes.voidptr_t);        // the value to append
                                
    Application.console.log("Done");
  },
  
  shutdown: function() {
    this.lib.close();
  },
  
  // Utility functions
  
  CFStringToJSString: function(cfstr) {
    var len = this.CFStringGetLength(cfstr);
    var buffer = ctypes.jschar.array(len+1)();
    
    this.CFStringGetCharacters(cfstr, CFRange(0, len), buffer);
    return buffer.readString();
  }
}

var Carbon = {
  init: function() {
    Application.console.log("Opening CoreFoundation");
    this.lib = ctypes.open("/System/Library/Frameworks/Carbon.framework/Carbon");
    
    // Types
    
    Application.console.log("Declaring miscellaneous types");
    
    this.struct_FSRef = new ctypes.StructType("FSRef",
                [ {"hidden": ctypes.char.array(80)}]);
        
    Application.console.log("Done.");
  },
  
  shutdown: function() {
    this.lib.close();
  }
}

var AppServices = {
  init: function() {
    Application.console.log("Opening Application Services");
    this.lib = ctypes.open("/System/Library/Frameworks/ApplicationServices.framework/ApplicationServices");
    
    // Launch services
    
    Application.console.log("Declaring Launch Services");
    
    this.kLSRolesNone = 1;
    this.kLSRolesViewer = 2;
    this.kLSRolesEditor = 4;
    this.kLSRolesAll = 0xffffffff;
    
    Application.console.log(" struct");
    this.struct_LSApplicationParameters = new ctypes.StructType('LSApplicationParameters',
                                              [ {'version': CFIndex},
                                                {'flags': OptionBits},
                                                {'application': ctypes.voidptr_t},  // FSRef of application to launch
                                                {'asyncLaunchRefCon': ctypes.voidptr_t},
                                                {'environment': ctypes.voidptr_t},  // CFDictionaryRef
                                                {'argv': ctypes.voidptr_t},         // CFArrayRef of args
                                                {'initialEvent': ctypes.voidptr_t}]); // AppleEvent *
   
    Application.console.log(" funcs");
    this.LSOpenURLsWithRole = this.lib.declare("LSOpenURLsWithRole",
                                ctypes.default_abi,                 // ABI type
                                OSStatus,                           // Returns OSStatus
                                CoreFoundation.CFArrayRef,          // Should be CoreFoundation.CFArrayRef
                                OptionBits,                         // Roles mask
                                ctypes.voidptr_t,                   // inAEParam
                                this.struct_LSApplicationParameters.ptr, // this.struct_LSApplicationParameters.ptr
                                ctypes.voidptr_t,                   // PSN array pointer
                                CFIndex);                           // max PSN count
    Application.console.log("Done.");
  },
  
  shutdown: function() {
    this.lib.close();
  }
}

var iPhoto = {
	// Install a timeout handler to install the interval routine

	startup: function() {
	  if (document.getElementById("contentAreaContextMenu")) {
	    document.getElementById("contentAreaContextMenu").addEventListener("popupshowing", iPhoto.onPopup, false);
	  }
	},
	
	onPopup: function() {
	  var node = iPhoto.getCurrentNode();
	  
	  var item = document.getElementById("add-to-iphoto_menuitem");
	  
	  if (item) {
	    item.hidden = (node == null); // Hide it if we're not on an image
	  }
  },
  
  add: function() {
    var node = iPhoto.getCurrentNode();
    
    if (node) {
      var src = node.getAttribute("src"); // Get the URL of the image
      if (src && src != "") {
        iPhoto.addImageByURL(src);
      }
    }
  },
  
  addImageByURL: function(src) {
    Application.console.log("Adding " + src);
    CoreFoundation.init();
    Carbon.init();
    AppServices.init();
    
    var array = CoreFoundation.CFArrayCreateMutable(null, 1, null);
    if (array) {
      Application.console.log("  Created array");
      var cfstr = CoreFoundation.CFStringCreateWithCharacters(null, src, src.length);
      var url = CoreFoundation.CFURLCreateWithString(null, cfstr, null);
      CoreFoundation.CFRelease(cfstr);
      CoreFoundation.CFArrayAppendValue(array, url);
      CoreFoundation.CFRelease(url);
      
      // Call Launch Services to open iPhoto and deliver the image
      
      var ref = new Carbon.struct_FSRef;
      var appParams = AppServices.struct_LSApplicationParameters(0, 1, ref.address(), null, null, null, null);
    
      var appstr = "file:///Applications/iPhoto.app";
      var appstrCF = CoreFoundation.CFStringCreateWithCharacters(null, appstr, appstr.length);
      var appurl = CoreFoundation.CFURLCreateWithString(null, appstrCF, null);
      CoreFoundation.CFRelease(appstrCF);
      var b = CoreFoundation.CFURLGetFSRef(appurl, ref.address());
      
      if (!b) {
        Application.console.log("Failed to get FSRef");
      } else {
        Application.console.log("Got the FSRef");
        Application.console.log("About to try to open");
        var status = AppServices.LSOpenURLsWithRole(array, 0, null, appParams.address(), null, 0);
        Application.console.log("Open result = " + status);
      }
      CoreFoundation.CFRelease(appurl);
            
      // Clean up
      
      CoreFoundation.CFRelease(array);
      Application.console.log("Done");
    } else {
      Application.console.log("  Unable to create the array");
    }
    
    AppServices.shutdown();
    Carbon.shutdown();
    CoreFoundation.shutdown();
  },
  
  getCurrentNode: function() {
    var node = document.popupNode;
    
    // If no node, just return null now
    
    if (node == undefined || !node) {
      return null;
    }
    
    // Is it an image node?
    
    var elemName = node.localName.toUpperCase();
    
    if (elemName == "IMG") {
      return node;
    }
    
    // Nope, return null
    
    return null;
  }
  
}
// Install load handler

window.addEventListener("load", function(e) { iPhoto.startup(); }, false);
